import keiko.flags


class TestFlags(object):

    def test_build_lamp_flags(self):
        flags = keiko.flags.build_lamp_flags(
            {'lamps': {'red': 'on', 'yellow': 'off'}})
        assert flags == '10XXXXXX'
        flags = keiko.flags.build_lamp_flags(
            {'lamps': {'yellow': 'blink', 'green': 'quickblink'}}
        )
        assert flags == 'X23XXXXX'

    def test_parse_lamp_flags(self):
        states = keiko.flags.parse_lamp_flags('10000000')
        assert states == {
            'lamps': {'red': 'on', 'yellow': 'off', 'green': 'off'}
        }
        states = keiko.flags.parse_lamp_flags('02300000')
        assert states == {
            'lamps': {'red': 'off', 'yellow': 'blink', 'green': 'quickblink'}
        }

    def test_lamp_state_interconversion(self):
        org = '12300000'
        state = keiko.flags.parse_lamp_flags(org)
        flags = keiko.flags.build_lamp_flags(state)
        assert flags == org.replace('0', 'X')

    def test_build_buzzer_flags(self):
        flags = keiko.flags.build_buzzer_flags({'buzzer': 'continuous'})
        assert flags == 'XXX1XXXX'

    def test_parse_buzzer_flags(self):
        states = keiko.flags.parse_buzzer_flags('00001000')
        assert states == {
            'buzzer': 'intermittent'
        }

    def test_buzzer_state_interconversion(self):
        org = '00010000'
        state = keiko.flags.parse_buzzer_flags(org)
        flags = keiko.flags.build_buzzer_flags(state)
        assert flags == org.replace('0', 'X')

    def test_build_do_flags(self):
        flags = keiko.flags.build_do_flags({'do': {1: 'on', 3: 'off'}})
        assert flags == '1X0XXXXX'

    def test_parse_do_flags(self):
        states = keiko.flags.parse_do_flags('10100000')
        assert states == {'do': {1: 'on', 2: 'off', 3: 'on', 4: 'off'}}

    def test_do_state_interconversion(self):
        org = '11110000'
        state = keiko.flags.parse_do_flags(org)
        flags = keiko.flags.build_do_flags(state)
        assert flags == org.replace('0', 'X')

    def test_parse_di_flags(self):
        states = keiko.flags.parse_di_flags('0101')
        assert states == {'di': {1: 'off', 2: 'on', 3: 'off', 4: 'on'}}

    def test_build_voice_flags(self):
        flags = keiko.flags.build_voice_flags(
            {'voice': {'number': 15, 'repeat': 8}}
        )
        assert flags == '11510800'
        flags = keiko.flags.build_voice_flags(
            {'voice': {'number': 2, 'repeat': 0}}
        )
        assert flags == '10200000'
        flags = keiko.flags.build_voice_flags({'voice': 'stop'})
        assert flags == '00000000'

    def test_parse_voice_flags(self):
        states = keiko.flags.parse_voice_flags('00000000')
        assert states == {'voice': 'stop'}
        states = keiko.flags.parse_voice_flags('10911000')
        assert states == {
            'voice': {'number': 9, 'repeat': 10}
        }

    def test_voice_state_interconversion(self):
        org = '11510800'
        state = keiko.flags.parse_voice_flags(org)
        flags = keiko.flags.build_voice_flags(state)
        assert flags == org



import sys

import mock
import pytest

import keiko.clients


class TestClient(object):

    def setup(self):
        self.client = keiko.clients.Client('127.0.0.1')  # dummy address
        self.client.raw._execute = mock.MagicMock()

    def teardown(self):
        del self.client

    def get_sent_command(self):
        args, kwargs = self.client.raw._execute.call_args
        return args[0]

    def set_received_message(self, message):
        self.client.raw._execute.return_value = message

    def test_get_lamps(self):
        self.set_received_message('00000000')
        self.client.lamps.status
        assert self.get_sent_command() == 'ACOP -u 1'

    def test_get_lamps_with_state(self):
        self.set_received_message('10100000')
        assert self.client.lamps.status == {
            'red': 'on', 'yellow': 'off', 'green': 'on'
        }

    def test_get_lamp(self):
        self.set_received_message('00000000')
        self.client.lamps.red.status
        assert self.get_sent_command() == 'ACOP -u 1'

    def test_get_lamp_with_state(self):
        self.set_received_message('10100000')
        assert self.client.lamps.red.status == 'on'
        assert self.client.lamps.yellow.status == 'off'
        assert self.client.lamps.green.status == 'on'

    def test_get_lamp_with_mode(self):
        self.set_received_message('02300000')
        assert self.client.lamps.red.status == 'off'
        assert self.client.lamps.yellow.status == 'blink'
        assert self.client.lamps.green.status == 'quickblink'

    def test_turn_on_lamp(self):
        self.client.lamps.red.on()
        assert self.get_sent_command() == 'ACOP -u 1 1XXXXXXX -w 0 -t 0'

    def test_turn_on_lamp_with_mode(self):
        self.client.lamps.yellow.blink()
        assert self.get_sent_command() == 'ACOP -u 1 X2XXXXXX -w 0 -t 0'
        self.client.lamps.green.quickblink()
        assert self.get_sent_command() == 'ACOP -u 1 XX3XXXXX -w 0 -t 0'

    def test_turn_on_lamp_with_wait_and_time(self):
        self.client.lamps.green.on(wait=2, time=3)
        assert self.get_sent_command() == 'ACOP -u 1 XX1XXXXX -w 2 -t 3'

    def test_turn_off_lamp(self):
        self.client.lamps.red.off()
        assert self.get_sent_command() == 'ACOP -u 1 0XXXXXXX -w 0 -t 0'

    def test_turn_off_lamp_with_wait(self):
        self.client.lamps.yellow.off(wait=2)
        assert self.get_sent_command() == 'ACOP -u 1 X0XXXXXX -w 2 -t 0'

    def test_turn_off_all_lamps(self):
        self.client.lamps.off()
        assert self.get_sent_command() == 'ACOP -u 1 000XXXXX -w 0 -t 0'

    def test_get_buzzer(self):
        self.set_received_message('00000000')
        self.client.buzzer.status
        assert self.get_sent_command() == 'ACOP -u 1'

    def test_get_buzzer_with_state(self):
        self.set_received_message('00000000')
        assert self.client.buzzer.status == 'off'

    def test_get_buzzer_with_mode(self):
        self.set_received_message('00010000')
        assert self.client.buzzer.status == 'continuous'
        self.set_received_message('00001000')
        assert self.client.buzzer.status == 'intermittent'

    def test_turn_on_buzzer(self):
        self.client.buzzer.on()
        assert self.get_sent_command() == 'ACOP -u 1 XXX1XXXX -w 0 -t 0'

    def test_turn_on_buzzer_with_mode(self):
        self.client.buzzer.intermittent()
        assert self.get_sent_command() == 'ACOP -u 1 XXXX1XXX -w 0 -t 0'

    def test_turn_on_buzzer_with_wait_and_time(self):
        self.client.buzzer.on(wait=2, time=3)
        assert self.get_sent_command() == 'ACOP -u 1 XXX1XXXX -w 2 -t 3'

    def test_turn_off_buzzer(self):
        self.client.buzzer.off()
        assert self.get_sent_command() == 'ACOP -u 1 XXX00XXX -w 0 -t 0'

    def test_turn_off_buzzer_with_wait(self):
        self.client.buzzer.off(wait=2)
        assert self.get_sent_command() == 'ACOP -u 1 XXX00XXX -w 2 -t 0'

    def test_get_dos(self):
        self.set_received_message('10100000')
        self.client.do.status
        assert self.get_sent_command() == 'ACOP -u 2'

    def test_get_dos_with_state(self):
        self.set_received_message('10100000')
        assert self.client.do.status == {
            1: 'on', 2: 'off', 3: 'on', 4: 'off'
        }

    def test_get_do(self):
        self.set_received_message('10100000')
        self.client.do(1).status
        assert self.get_sent_command() == 'ACOP -u 2'

    def test_get_do_with_state(self):
        self.set_received_message('10100000')
        assert self.client.do(1).status == 'on'
        assert self.client.do(2).status == 'off'
        assert self.client.do(3).status == 'on'
        assert self.client.do(4).status == 'off'

    def test_turn_on_do(self):
        self.client.do(1).on()
        assert self.get_sent_command() == 'ACOP -u 2 1XXXXXXX -w 0 -t 0'

    def test_turn_on_do_with_wait_and_time(self):
        self.client.do(2).on(wait=3, time=4)
        assert self.get_sent_command() == 'ACOP -u 2 X1XXXXXX -w 3 -t 4'

    def test_turn_off_do(self):
        self.client.do(3).off()
        assert self.get_sent_command() == 'ACOP -u 2 XX0XXXXX -w 0 -t 0'

    def test_turn_off_do_with_wait(self):
        self.client.do(4).off(wait=5)
        assert self.get_sent_command() == 'ACOP -u 2 XXX0XXXX -w 5 -t 0'

    def test_get_dis(self):
        self.set_received_message('0101')
        self.client.di.status
        assert self.get_sent_command() == 'ROPS'

    def test_get_dis_with_state(self):
        self.set_received_message('0101')
        assert self.client.di.status == {
            1: 'off', 2: 'on', 3: 'off', 4: 'on'
        }

    def test_get_di(self):
        self.set_received_message('0101')
        self.client.di(1).status
        assert self.get_sent_command() == 'ROPS'

    def test_get_di_with_state(self):
        self.set_received_message('0101')
        assert self.client.di(1).status == 'off'
        assert self.client.di(2).status == 'on'
        assert self.client.di(3).status == 'off'
        assert self.client.di(4).status == 'on'

    def test_get_voices(self):
        self.set_received_message('00911010')
        self.client.voices.status
        assert self.get_sent_command() == 'SPOP'

    def test_get_voices_with_state(self):
        self.set_received_message('00911010')
        assert self.client.voices.status == 'stop'
        self.set_received_message('10911010')
        assert self.client.voices.status == {'number': 9, 'repeat': 10}

    def test_get_voice(self):
        self.set_received_message('00911010')
        self.client.voices(9).status
        assert self.get_sent_command() == 'SPOP'

    def test_get_voice_with_state(self):
        self.set_received_message('00911010')
        assert self.client.voices(9).status == 'stop'
        self.set_received_message('10911010')
        assert self.client.voices(9).status == 'play'
        assert self.client.voices(10).status == 'stop'

    def test_play_voice(self):
        self.client.voices(1).play()
        assert self.get_sent_command() == 'SPOP 10110100'

    def test_play_voice_with_times(self):
        self.client.voices(10).play(times=20)
        assert self.get_sent_command() == 'SPOP 11012000'

    def test_repeat_voice(self):
        self.client.voices(1).repeat()
        assert self.get_sent_command() == 'SPOP 10100000'

    def test_stop_voices(self):
        self.client.voices.stop()
        assert self.get_sent_command() == 'SPOP 00000000'

    def test_stop_voice(self):
        self.client.voices(20).stop()
        assert self.get_sent_command() == 'SPOP 00000000'


class TestRawClient(object):

    address = '127.0.0.1'
    port = 60000

    def setup(self):
        self.patcher = mock.patch('socket.socket')
        self.patcher.start()
        self.client = keiko.clients.RawClient(self.address, self.port)
        self.client._strip_data = mock.Mock()  # has trouble on py3

    def teardown(self):
        del self.client
        self.patcher.stop()

    def get_sent_data(self):
        args, kwargs = self.client._sock.sendall.call_args
        data = args[0]
        if sys.version_info[0] >= 3:  # py3
            return str(data, 'utf-8')  # bytes to str
        else:  # py2
            return data

    def test_connect(self):
        self.client.help()
        self.client._sock.connect.assert_called_with(
            (self.address, self.port)
        )

    def test_error(self):
        self.client._send = mock.Mock(return_value='ER01')  # error code
        with pytest.raises(Exception):
            self.client.help()

    def test_acop_read(self):
        self.client.acop()
        assert self.get_sent_data() == 'ACOP -u 1\r'

    def test_acop_write_with_default(self):
        self.client.acop('X12XXXXX')
        assert self.get_sent_data() == 'ACOP -u 1 X12XXXXX -w 0 -t 0\r'

    def test_acop_write_with_param(self):
        self.client.acop('101XXXXX', unit=2, wait=3, time=4)
        assert self.get_sent_data() == 'ACOP -u 2 101XXXXX -w 3 -t 4\r'

    def test_alof_write(self):
        self.client.alof()
        assert self.get_sent_data() == 'ALOF\r'

    def test_ckdi_read(self):
        self.client.ckdi()
        assert self.get_sent_data() == 'CKDI\r'

    def test_ckdi_write(self):
        self.client.ckdi('EDEX')
        assert self.get_sent_data() == 'CKDI EDEX\r'

    def test_ckid_read(self):
        self.client.ckid()
        assert self.get_sent_data() == 'CKID\r'

    def test_ckid_write(self):
        self.client.ckid('Enable')
        assert self.get_sent_data() == 'CKID Enable\r'

    def test_ckip_read(self):
        self.client.ckip()
        assert self.get_sent_data() == 'CKIP\r'

    def test_ckip_write(self):
        self.client.ckip('EDEXXXXXXXXXXXXXXXXX')
        assert self.get_sent_data() == 'CKIP EDEXXXXXXXXXXXXXXXXX\r'

    def test_ckst_read(self):
        self.client.ckst()
        assert self.get_sent_data() == 'CKST\r'

    def test_help_read(self):
        self.client.help()
        assert self.get_sent_data() == 'HELP\r'

    def test_lgpw_read(self):
        self.client.lgpw()
        assert self.get_sent_data() == 'LGPW\r'

    def test_lgpw_write(self):
        self.client.lgpw('isa')
        assert self.get_sent_data() == 'LGPW isa\r'

    def test_pwst_read(self):
        self.client.pwst()
        assert self.get_sent_data() == 'PWST\r'

    def test_pwst_write(self):
        self.client.pwst('Enable')
        assert self.get_sent_data() == 'PWST Enable\r'

    def test_rdcd_read(self):
        self.client.rdcd()
        assert self.get_sent_data() == 'RDCD\r'

    def test_rdcn_read(self):
        self.client.rdcn()
        assert self.get_sent_data() == 'RDCN\r'

    def test_rdmn_read(self):
        self.client.rdmn()
        assert self.get_sent_data() == 'RDMN\r'

    def test_rdpd_read(self):
        self.client.rdpd()
        assert self.get_sent_data() == 'RDPD\r'

    def test_rdsn_read(self):
        self.client.rdsn()
        assert self.get_sent_data() == 'RDSN\r'

    def test_rly1_read(self):
        self.client.rly1()
        assert self.get_sent_data() == 'RLY1\r'

    def test_rly2_read(self):
        self.client.rly2()
        assert self.get_sent_data() == 'RLY2\r'

    def test_rly3_read(self):
        self.client.rly3()
        assert self.get_sent_data() == 'RLY3\r'

    def test_rly4_read(self):
        self.client.rly4()
        assert self.get_sent_data() == 'RLY4\r'

    def test_rly5_read(self):
        self.client.rly5()
        assert self.get_sent_data() == 'RLY5\r'

    def test_rly6_read(self):
        self.client.rly6()
        assert self.get_sent_data() == 'RLY6\r'

    def test_rly7_read(self):
        self.client.rly7()
        assert self.get_sent_data() == 'RLY7\r'

    def test_rly8_read(self):
        self.client.rly8()
        assert self.get_sent_data() == 'RLY8\r'

    def test_rly_write_with_default(self):
        self.client.rly1('TurnOn')
        assert self.get_sent_data() == 'RLY1 TurnOn -w 0 -t 0\r'

    def test_rly_write_with_param(self):
        self.client.rly2('Blink', wait=10, time=30)
        assert self.get_sent_data() == 'RLY2 Blink -w 10 -t 30\r'

    def test_rops_read(self):
        self.client.rops()
        assert self.get_sent_data() == 'ROPS\r'

    def test_ryin_read(self):
        self.client.ryin(1)
        assert self.get_sent_data() == 'RYIN -n 1\r'

    def test_ryof_write(self):
        self.client.ryof(2)
        assert self.get_sent_data() == 'RYOF -n 2\r'

    def test_ryot_read(self):
        self.client.ryot(2)
        assert self.get_sent_data() == 'RYOT -n 2\r'

    def test_ryot_write_with_default(self):
        self.client.ryot(1, 'TurnOn')
        assert self.get_sent_data() == 'RYOT -n 1 TurnOn -w 0 -t 0\r'

    def test_ryot_write_with_param(self):
        self.client.ryot(2, 'TurnOn', wait=10, time=20)
        assert self.get_sent_data() == 'RYOT -n 2 TurnOn -w 10 -t 20\r'

    def test_spop_read(self):
        self.client.spop()
        assert self.get_sent_data() == 'SPOP\r'

    def test_spop_write(self):
        self.client.spop('10100000')
        assert self.get_sent_data() == 'SPOP 10100000\r'

    def test_utid_read(self):
        self.client.utid()
        assert self.get_sent_data() == 'UTID\r'

    def test_vern_read(self):
        self.client.vern()
        assert self.get_sent_data() == 'VERN\r'



"""
Provides Web API server for Keiko-chan.
"""

from flask import Flask, request, jsonify, abort

from keiko.clients import Client


app = Flask(__name__)


_VALID_COLORS = ['green', 'yellow', 'red']
_VALID_TERMS = ['1', '2', '3', '4']


@app.route('/')
def index():
    return 'keiko.py API server'


@app.route('/lamps')
def get_all_lamps():
    return jsonify(lamps=app.keiko.lamps.status)


@app.route('/lamps/<color>')
def get_lamp(color):
    if color not in _VALID_COLORS:
        abort(400)
    lamp = getattr(app.keiko.lamps, color)
    return jsonify(lamps={color: lamp.status})


@app.route('/lamps/<color>/<state>')
def set_lamp(color, state):
    if color not in _VALID_COLORS:
        abort(400)
    lamp = getattr(app.keiko.lamps, color)
    wait = request.args.get('wait', 0)
    time = request.args.get('time', 0)
    if state in ['on', 'blink', 'quickblink']:
        getattr(lamp, state)(wait, time)
    elif state == 'off':
        lamp.off(wait)
    else:
        abort(400)
    return jsonify(result='success')


@app.route('/lamps/off')
def set_all_lamps_off():
    wait = request.args.get('wait', 0)
    app.keiko.lamps.off(wait)
    return jsonify(result='success')


@app.route('/buzzer')
def get_buzzer():
    return jsonify(buzzer=app.keiko.buzzer.status)


@app.route('/buzzer/<state>')
def set_buzzer(state):
    wait = request.args.get('wait', 0)
    time = request.args.get('time', 0)
    if state in ['on', 'continuous', 'intermittent']:
        getattr(app.keiko.buzzer, state)(wait, time)
    elif state == 'off':
        app.keiko.buzzer.off(wait)
    else:
        abort(400)
    return jsonify(result='success')


@app.route('/do')
def get_all_dos():
    return jsonify(do=app.keiko.do.status)


@app.route('/do/<term>')
def get_do(term):
    if term not in _VALID_TERMS:
        abort(400)
    do = app.keiko.do(int(term))
    return jsonify(do={term: do.status})


@app.route('/do/<term>/<state>')
def set_do(term, state):
    if term not in _VALID_TERMS:
        abort(400)
    do = app.keiko.do(int(term))
    wait = request.args.get('wait', 0)
    time = request.args.get('time', 0)
    if state == 'on':
        do.on(wait, time)
    elif state == 'off':
        do.off(wait)
    else:
        abort(400)
    return jsonify(result='success')


@app.route('/di')
def get_all_dis():
    return jsonify(di=app.keiko.di.status)


@app.route('/di/<term>')
def get_di(term):
    if term not in _VALID_TERMS:
        abort(400)
    di = app.keiko.di(int(term))
    return jsonify(di={term: di.status})


@app.route('/voices')
def get_all_voices():
    return jsonify(voices=app.keiko.voices.status)


@app.route('/voices/<number>')
def get_voice(number):
    if not (number.isdigit() and 1 <= int(number) <= 20):
        abort(400)
    voice = app.keiko.voices(int(number))
    return jsonify(voices={number: voice.status})


@app.route('/voices/<number>/<state>')
def set_voice(number, state):
    if not (number.isdigit() and 1 <= int(number) <= 20):
        abort(400)
    voice = app.keiko.voices(int(number))
    times = request.args.get('times', 1)
    if state == 'play':
        voice.play(times)
    elif state in ['repeat', 'stop']:
        getattr(voice, state)()
    else:
        abort(400)
    return jsonify(result='success')


@app.route('/voices/stop')
def set_all_voices_stop():
    app.keiko.voices.stop()
    return jsonify(result='success')


@app.route('/contract')
def get_contract():
    return jsonify(contract={
        'deadline': app.keiko.raw.rdcd(),
        'number': app.keiko.raw.rdcn()
    })


@app.route('/model')
def get_model():
    return jsonify(model=app.keiko.raw.rdmn())


@app.route('/productiondate')
def get_productiondate():
    return jsonify(productiondate=app.keiko.raw.rdpd())


@app.route('/serialnumber')
def get_serialnumber():
    return jsonify(serialnumber=app.keiko.raw.rdsn())


@app.route('/unitid')
def get_unitid():
    return jsonify(unitid=app.keiko.raw.utid())


@app.route('/version')
def get_version():
    return jsonify(version=app.keiko.raw.vern())


def main():
    import argparse
    import logging
    import os

    parser = argparse.ArgumentParser()
    parser.add_argument(
        'address',
        metavar='ADDRESS',
        help='address of Keiko-chan'
    )
    parser.add_argument(
        '--port',
        default=60000,
        help='port of Keiko-chan[60000]'
    )
    parser.add_argument(
        '--server',
        default='127.0.0.1:8080',
        help='address and port of API server[127.0.0.1:8080]'
    )
    parser.add_argument(
        '--debug',
        type=bool,
        default=False,
        help='run API server on debug mode[False]'
    )
    args = parser.parse_args()

    log_handler = logging.FileHandler(
        os.path.join(os.getcwd(), 'error.log')
    )
    log_handler.setLevel(logging.ERROR)
    app.logger.addHandler(log_handler)

    app.keiko = Client(args.address, args.port)
    app.debug = args.debug
    host, port = args.server.split(':')
    app.run(host=host, port=int(port))



import sys

import mock

import keiko.app


class TestApp(object):
    """Smoke test only."""

    def setup(self):
        keiko.app.app.keiko = mock.MagicMock()
        keiko.app.jsonify = mock.MagicMock(return_value='')
        self.app = keiko.app.app.test_client()

    def test_index(self):
        assert self.app.get('/').status_code == 200

    def test_get_all_lamps(self):
        assert self.app.get('/lamps').status_code == 200

    def test_get_lamp(self):
        assert self.app.get('/lamps/red').status_code == 200
        assert self.app.get('/lamps/yellow').status_code == 200
        assert self.app.get('/lamps/green').status_code == 200
        assert self.app.get('/lamps/blue').status_code == 400

    def test_set_lamp(self):
        assert self.app.get('/lamps/red/on').status_code == 200
        assert self.app.get('/lamps/yellow/blink').status_code == 200
        assert self.app.get('/lamps/green/quickblink').status_code == 200
        assert self.app.get('/lamps/red/off').status_code == 200
        assert self.app.get('/lamps/blue/on').status_code == 400
        assert self.app.get('/lamps/red/light').status_code == 400

    def test_set_all_lamps_off(self):
        assert self.app.get('/lamps/off').status_code == 200

    def test_get_buzzer(self):
        assert self.app.get('/buzzer').status_code == 200

    def test_set_buzzer(self):
        assert self.app.get('/buzzer/on').status_code == 200
        assert self.app.get('/buzzer/continuous').status_code == 200
        assert self.app.get('/buzzer/intermittent').status_code == 200
        assert self.app.get('/buzzer/off').status_code == 200
        assert self.app.get('/buzzer/beep').status_code == 400

    def test_get_all_dos(self):
        assert self.app.get('/do').status_code == 200

    def test_get_do(self):
        assert self.app.get('/do/1').status_code == 200
        assert self.app.get('/do/2').status_code == 200
        assert self.app.get('/do/3').status_code == 200
        assert self.app.get('/do/4').status_code == 200
        assert self.app.get('/do/0').status_code == 400
        assert self.app.get('/do/5').status_code == 400

    def test_set_do(self):
        assert self.app.get('/do/1/on').status_code == 200
        assert self.app.get('/do/2/off').status_code == 200
        assert self.app.get('/do/0/on').status_code == 400
        assert self.app.get('/do/1/blink').status_code == 400

    def test_get_all_dis(self):
        assert self.app.get('/di').status_code == 200

    def test_get_di(self):
        assert self.app.get('/di/1').status_code == 200
        assert self.app.get('/di/2').status_code == 200
        assert self.app.get('/di/3').status_code == 200
        assert self.app.get('/di/4').status_code == 200
        assert self.app.get('/di/0').status_code == 400
        assert self.app.get('/di/5').status_code == 400

    def test_get_all_voices(self):
        assert self.app.get('/voices').status_code == 200

    def test_get_voice(self):
        assert self.app.get('/voices/1').status_code == 200
        assert self.app.get('/voices/20').status_code == 200
        assert self.app.get('/voices/0').status_code == 400
        assert self.app.get('/voices/21').status_code == 400

    def test_set_voice(self):
        assert self.app.get('/voices/1/play').status_code == 200
        assert self.app.get('/voices/2/repeat').status_code == 200
        assert self.app.get('/voices/3/stop').status_code == 200
        assert self.app.get('/voices/0/play').status_code == 400
        assert self.app.get('/voices/1/speak').status_code == 400

    def test_set_all_voices_stop(self):
        assert self.app.get('/voices/stop').status_code == 200

    def test_get_contract(self):
        assert self.app.get('/contract').status_code == 200

    def test_get_model(self):
        assert self.app.get('/model').status_code == 200

    def test_get_productiondate(self):
        assert self.app.get('/productiondate').status_code == 200

    def test_get_serialnumber(self):
        assert self.app.get('/serialnumber').status_code == 200

    def test_get_unitid(self):
        assert self.app.get('/unitid').status_code == 200

    def test_get_version(self):
        assert self.app.get('/version').status_code == 200


class TestMain(object):

    def setup(self):
        self._argv = sys.argv
        sys.argv = []

    def teardown(self):
        sys.argv = self._argv

    def test_main_with_default(self):
        with mock.patch('keiko.app.app.run') as m:
            sys.argv.extend(['script_path', 'keiko_address'])
            keiko.app.main()
            assert keiko.app.app.keiko is not None
            assert keiko.app.app.debug is False
            kwargs = m.call_args[1]
            assert kwargs == {'host': '127.0.0.1', 'port': 8080}



"""
Provides utilities for flags arguments of commands.
"""

# lamps
_LAMP_DIGITS = {
    'red': 0,
    'yellow': 1,
    'green': 2
}
_LAMP_STATES = {
    'off': 0,
    'on': 1,
    'blink': 2,
    'quickblink': 3
}


def build_lamp_flags(states):
    """Returns flags that be used as arguments of lamp control commands.

    The structure of <states> is:

        {'lamps': {<color>: <state>}}

    <color> is red, yellow, or green.
    <state> is on, blink, quickblink, or off.

    Example:

        {'lamps': {'red': 'on', 'green': 'off'}}
    """
    flag_list = list('XXXXXXXX')
    for color, state in states['lamps'].items():
        flag_list[_LAMP_DIGITS[color]] = str(_LAMP_STATES[state])
    return ''.join(flag_list)


def parse_lamp_flags(flags):
    """Parses flags and returns a dict that represents the lamp states."""
    # flags: [0123]{8}
    values = _swap_key_and_value(_LAMP_STATES)  # {value: state}
    states = dict([
        (color, values[flags[digit]]) for color, digit in _LAMP_DIGITS.items()
    ])
    return {'lamps': states}


# buzzer
_BUZZER_DIGITS = {
    'continuous': 3,
    'intermittent': 4
}


def build_buzzer_flags(states):
    """Returns flags that be used as arguments of buzzer control commands.

    The structure of <states> is:

        {'buzzer': <state>}

    <state> is continuous, intermittent, or off.

    Example:

        {'buzzer': 'continuous'}
    """
    flag_list = list('XXXXXXXX')
    state = states['buzzer']
    if state == 'off':
        for digit in _BUZZER_DIGITS.values():
            flag_list[digit] = '0'
    else:
        flag_list[_BUZZER_DIGITS[state]] = '1'
    return ''.join(flag_list)


def parse_buzzer_flags(flags):
    """Parses flags and returns a dict that represents the buzzer states."""
    # flags: [0123]{8}
    for state, digit in _BUZZER_DIGITS.items():
        if flags[digit] == '1':
            return {'buzzer': state}
    return {'buzzer': 'off'}


# DOs
_DO_DIGITS = {  # {term: digit}
    1: 0,
    2: 1,
    3: 2,
    4: 3
}
_DO_STATES = {
    'off': 0,
    'on': 1
}


def build_do_flags(states):
    """Returns flags that be used as arguments of DOs control commands.

    The structure of <states> is:

        {'do': {<term>: <state>}}

    <term> is a terminal number. <state> is on or off.

    Example:

        {'do': {1: 'on', 3: 'off'}}
    """
    flag_list = list('XXXXXXXX')
    for term, state in states['do'].items():
        flag_list[_DO_DIGITS[term]] = str(_DO_STATES[state])
    return ''.join(flag_list)


def parse_do_flags(flags):
    """Parses flags and returns a dict that represents the DOs states."""
    # flags: [01]{8}
    values = _swap_key_and_value(_DO_STATES)  # {value: state}
    states = dict([
        (term, values[flags[digit]]) for term, digit in _DO_DIGITS.items()
    ])
    return {'do': states}


# DIs
_DI_DIGITS = {  # {term: digit}
    1: 0,
    2: 1,
    3: 2,
    4: 3
}
_DI_STATES = {
    'off': 0,
    'on': 1
}


def parse_di_flags(flags):
    """Parses flags and returns a dict that represents the DIs states."""
    # flags: [01]{4}
    values = _swap_key_and_value(_DI_STATES)  # {value: state}
    states = dict([
        (term, values[flags[digit]]) for term, digit in _DI_DIGITS.items()
    ])
    return {'di': states}


# voice
def build_voice_flags(states):
    """Returns flags that be used as arguments of voice control commands.

    The structure of <states> is:

        {'voice': <state>}

    <state> is stop, or {'number': <number>, 'repeat': <repeat>}.
    <number> is voice number to play, 1 - 20.
    <repeat> is total number of plays. You can set 0 to repeat infinitely.

    Example:

        {'voice: {'number': 1, 'repeat': 3}}
    """
    if states['voice'] == 'stop':
        return '00000000'
    number = states['voice']['number']
    repeat = states['voice']['repeat']
    if repeat == 0:
        return '1{0:02}00000'.format(number)  # to repeat infinitely
    else:
        return '1{0:02}1{1:02}00'.format(number, repeat)


def parse_voice_flags(flags):
    """Parses flags and returns a dict that represents voice playing state."""
    # flags: [0-9]{8}
    if flags[0] == '0':
        return {'voice': 'stop'}
    else:
        return {'voice': {
            'number': int(flags[1:3]),
            'repeat': int(flags[4:6])
        }}


# inner utils
def _swap_key_and_value(dictionary):
    return dict((str(v), s) for s, v in dictionary.items())



from setuptools import setup
from setuptools.command.test import test as TestCommand

import keiko


class PyTest(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        pytest.main(self.test_args)


setup(
    name='keiko',
    version=keiko.__version__,
    description='keiko is Python and Web API clients for Keiko-chan.',
    long_description=open('README.rst').read(),
    author='lanius',
    author_email='lanius@nirvake.org',
    url='https://github.com/lanius/keiko/',
    packages=['keiko'],
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'keiko = keiko.app:main',
        ],
    },
    install_requires=['flask'],
    license=open('LICENSE').read(),
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: Japanese',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3'
    ),
    tests_require=['pytest', 'mock'],
    cmdclass = {'test': PyTest},
)



"""
Provides client classes for Keiko-chan.
"""

import collections
import contextlib
import socket
import sys

from keiko.flags import (
    build_lamp_flags, parse_lamp_flags,
    build_buzzer_flags, parse_buzzer_flags,
    build_do_flags, parse_do_flags,
    parse_di_flags,
    build_voice_flags, parse_voice_flags
)


class Client(object):
    """Provides high level APIs to control Keiko-chan."""

    def __init__(self, address, port=60000):
        self.raw = RawClient(address, port)
        self.lamps = LampHolder(self.raw)
        self.buzzer = Buzzer(self.raw)
        self.do = DOHolder(self.raw)
        self.di = DIHolder(self.raw)
        self.voices = VoiceHolder(self.raw)


class LampHolder(object):
    """Holds the lamps."""

    def __init__(self, rawclient):
        self.raw = rawclient
        self.red = Lamp(self.raw, 'red')
        self.yellow = Lamp(self.raw, 'yellow')
        self.green = Lamp(self.raw, 'green')

    @property
    def status(self):
        """Returns all the lamps state."""
        flags = self.raw.acop()
        states = parse_lamp_flags(flags)
        return states['lamps']

    def off(self, wait=0):
        """Turns off all the lamps."""
        flags = build_lamp_flags(
            {'lamps': {'red': 'off', 'yellow': 'off', 'green': 'off'}}
        )
        self.raw.acop(flags, wait=wait)


class Lamp(object):
    """A client to control the lamp."""

    def __init__(self, rawclient, color):
        self.raw = rawclient
        self.color = color

    @property
    def status(self):
        """Returns the lamp state."""
        flags = self.raw.acop()
        states = parse_lamp_flags(flags)
        return states['lamps'][self.color]

    def _set(self, state, wait=0, time=0):
        flags = build_lamp_flags({'lamps': {self.color: state}})
        self.raw.acop(flags, wait=wait, time=time)

    def on(self, wait=0, time=0):
        """Turns on the lamp."""
        self._set('on', wait, time)

    def blink(self, wait=0, time=0):
        """Blinks the lamp on and off."""
        self._set('blink', wait, time)

    def quickblink(self, wait=0, time=0):
        """Blinks the lamp on and off quickly."""
        self._set('quickblink', wait, time)

    def off(self, wait=0):
        """Turns off the lamp."""
        self._set('off', wait)


class Buzzer(object):
    """A client to control the buzzer."""

    def __init__(self, rawclient):
        self.raw = rawclient

    @property
    def status(self):
        """Returns the buzzer state."""
        flags = self.raw.acop()
        states = parse_buzzer_flags(flags)
        return states['buzzer']

    def _set(self, state, wait=0, time=0):
        flags = build_buzzer_flags({'buzzer': state})
        self.raw.acop(flags, wait=wait, time=time)

    def on(self, wait=0, time=0):
        """Turns on the buzzer to beep continuously."""
        self.continuous(wait, time)

    def continuous(self, wait=0, time=0):
        """Turns on the buzzer to beep continuously."""
        self._set('continuous', wait, time)

    def intermittent(self, wait=0, time=0):
        """Turns on the buzzer to beep intermittently."""
        self._set('intermittent', wait, time)

    def off(self, wait=0):
        """Turns off the buzzer."""
        self._set('off', wait)


class DOHolder(object):
    """Holds the DOs."""

    def __init__(self, rawclient):
        self.raw = rawclient

    def __call__(self, term):
        return DO(self.raw, term)

    @property
    def status(self):
        """Returns all the DOs state."""
        flags = self.raw.acop(unit=2)
        states = parse_do_flags(flags)
        return states['do']


class DO(object):
    """A client to control the direct output."""

    def __init__(self, rawclient, term):
        self.raw = rawclient
        self.term = term

    @property
    def status(self):
        """Returns the DO state."""
        flags = self.raw.acop(unit=2)
        states = parse_do_flags(flags)
        return states['do'][self.term]

    def _set(self, state, wait=0, time=0):
        flags = build_do_flags({'do': {self.term: state}})
        self.raw.acop(flags, unit=2, wait=wait, time=time)

    def on(self, wait=0, time=0):
        """Turns on the DO."""
        self._set('on', wait, time)

    def off(self, wait=0):
        """Turns off the DO."""
        self._set('off', wait)


class DIHolder(object):
    """Holds the DIs."""

    def __init__(self, rawclient):
        self.raw = rawclient

    def __call__(self, term):
        return DI(self.raw, term)

    @property
    def status(self):
        """Returns all the DIs state."""
        flags = self.raw.rops()
        states = parse_di_flags(flags)
        return states['di']


class DI(object):
    """A client to control the direct input."""

    def __init__(self, rawclient, term):
        self.raw = rawclient
        self.term = term

    @property
    def status(self):
        """Returns the DI state."""
        flags = self.raw.rops()
        states = parse_di_flags(flags)
        return states['di'][self.term]


class VoiceHolder(object):
    """Holds the voices."""

    def __init__(self, rawclient):
        self.raw = rawclient

    def __call__(self, number):
        return Voice(self.raw, number)

    @property
    def status(self):
        """Returns the voices state."""
        flags = self.raw.spop()
        states = parse_voice_flags(flags)
        return states['voice']

    def stop(self):
        """Stops playing any voice."""
        flags = build_voice_flags({'voice': 'stop'})
        self.raw.spop(flags)


class Voice(object):

    def __init__(self, rawclient, number):
        self.raw = rawclient
        self.number = number

    @property
    def status(self):
        """Returns the voice state."""
        flags = self.raw.spop()
        states = parse_voice_flags(flags)
        state = states['voice']
        if state == 'stop':
            return 'stop'
        if state['number'] == self.number:
            return 'play'
        else:
            return 'stop'

    def _set(self, state):
        flags = build_voice_flags({'voice': state})
        self.raw.spop(flags)

    def play(self, times=1):
        """Plays the voice."""
        self._set({'number': self.number, 'repeat': times})

    def repeat(self):
        """Plays the voice repeatedly."""
        self.play(0)

    def stop(self):
        """Stops playing the voice."""
        self._set('stop')


class RawClient(object):
    """Provides low level APIs to control Keiko-chan."""

    def __init__(self, address, port=60000):
        self.address = address
        self.port = port

    def _send(self, command):
        ret = ''
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        with contextlib.closing(self._sock):
            self._sock.connect((self.address, self.port))
            self._sock.sendall(self._build_data(command))
            ret = self._sock.recv(64)  # enough long
        return self._strip_data(ret)

    def _build_data(self, command):
        data = ''.join([command, '\r'])
        if sys.version_info[0] >= 3:  # py3
            return bytes(data, 'ascii')  # str to bytes
        else:  # py2
            return data

    def _strip_data(self, data):
        if sys.version_info[0] >= 3:  # py3
            data = str(data, 'utf-8')  # bytes to str
        return data.rstrip('\r')

    def _execute(self, command):
        result = self._send(command)
        errors = {
            'ER01': 'Invalid command',
            'ER02': 'Wrong EOL code',
            'ER03': 'Wrong arguments',
            'ER04': 'Command failed'
        }
        if result in errors:
            raise Exception(errors[result])
        return result

    def acop(self, flags=None, unit=1, wait=0, time=0):
        # flags: [0123X]{8}
        command = ' '.join(['ACOP', '-u', str(unit)])
        if flags:
            args = ' '.join([flags, '-w', str(wait), '-t', str(time)])
            return self._execute(' '.join([command, args]))
        else:
            return self._execute(command)

    def alof(self):
        return self._execute('ALOF')

    def ckdi(self, flags=None):
        # flags: [EDX]{4}
        command = 'CKDI'
        if flags:
            return self._execute(' '.join([command, flags]))
        else:
            return self._execute(command)

    def ckid(self, param=None):
        # param: Enable|Disable
        command = 'CKID'
        if param:
            return self._execute(' '.join([command, param]))
        else:
            return self._execute(command)

    def ckip(self, flags=None):
        # flags: [EDX]{20}
        command = 'CKIP'
        if flags:
            return self._execute(' '.join([command, flags]))
        else:
            return self._execute(command)

    def ckst(self):
        return self._execute('CKST')

    def help(self):
        return self._execute('HELP')

    def lgpw(self, new_password=None):
        command = 'LGPW'
        if new_password:
            return self._execute(' '.join([command, new_password]))
        else:
            return self._execute(command)

    def pwst(self, param=None):
        # param: Enable|Disable
        command = 'PWST'
        if param:
            return self._execute(' '.join([command, param]))
        else:
            return self._execute(command)

    def rdcd(self):
        return self._execute('RDCD')

    def rdcn(self):
        return self._execute('RDCN')

    def rdmn(self):
        return self._execute('RDMN')

    def rdpd(self):
        return self._execute('RDPD')

    def rdsn(self):
        return self._execute('RDSN')

    def _rly(self, command, param, wait, time):
        # command: RLY[1-8]
        # param: TurnOff|TurnOn|Blink
        if param:
            args = ' '.join([param, '-w', str(wait), '-t', str(time)])
            return self._execute(' '.join([command, args]))
        else:
            return self._execute(command)

    def rly1(self, param=None, wait=0, time=0):
        return self._rly('RLY1', param, wait, time)

    def rly2(self, param=None, wait=0, time=0):
        return self._rly('RLY2', param, wait, time)

    def rly3(self, param=None, wait=0, time=0):
        return self._rly('RLY3', param, wait, time)

    def rly4(self, param=None, wait=0, time=0):
        return self._rly('RLY4', param, wait, time)

    def rly5(self, param=None, wait=0, time=0):
        return self._rly('RLY5', param, wait, time)

    def rly6(self, param=None, wait=0, time=0):
        return self._rly('RLY6', param, wait, time)

    def rly7(self, param=None, wait=0, time=0):
        return self._rly('RLY7', param, wait, time)

    def rly8(self, param=None, wait=0, time=0):
        return self._rly('RLY8', param, wait, time)

    def rops(self):
        return self._execute('ROPS')

    def ryin(self, term):
        # term: [1-4]
        return self._execute(' '.join(['RYIN', '-n', str(term)]))

    def ryof(self, term):
        # term: [1-4]
        return self._execute(' '.join(['RYOF', '-n', str(term)]))

    def ryot(self, term, param=None, wait=0, time=0):
        command = 'RYOT'
        if param:  # TurnOff|TurnOn|Pulse
            args = ' '.join([
                '-n', str(term), param, '-w', str(wait), '-t', str(time)
            ])
            return self._execute(' '.join([command, args]))
        else:
            return self._execute(' '.join([command, '-n', str(term)]))

    def spop(self, flags=None):
        # flags: [0-9]{7}
        command = 'SPOP'
        if flags:
            return self._execute(' '.join([command, flags]))
        else:
            return self._execute(command)

    def utid(self):
        return self._execute('UTID')

    def vern(self):
        return self._execute('VERN')
